@@ -1,8 +1,8 @@
 /*
-  $Id: Tagger.h 13736 2011-12-20 16:51:18Z sloot $
+  $Id: Tagger.h 14402 2012-03-07 14:04:57Z sloot $
   $URL: https://ilk.uvt.nl/svn/trunk/sources/Mbt3/include/mbt/Tagger.h $
 
-  Copyright (c) 1998 - 2011
+  Copyright (c) 1998 - 2012
   ILK   - Tilburg University
   CLiPS - University of Antwerp
  
@@ -72,7 +72,9 @@ namespace Tagger {
     double *path_prob;
     n_best_tuple **n_best_array;
   };
-  
+
+  class TagResult;
+
   class TaggerClass{
   public:
     TaggerClass( );
@@ -83,16 +85,22 @@ namespace Tagger {
     bool InitBeaming( unsigned int );
     TaggerClass *clone() const;
     int Run( );
-    int TagLine( const std::string&, std::string& );
-    std::string Tag( const std::string& );
+    std::vector<TagResult> tagLine( const std::string& );
+    std::vector<TagResult> tagSentence( sentence& );
+    std::string Tag( const std::string& inp ){
+      return TRtoString( tagLine(inp) );
+    };
+    std::string TRtoString( const std::vector<TagResult>& ) const;
+    int TagLine( const std::string&, std::string& ); 
+    // only for backward compatability
     int CreateKnown();
     int CreateUnknown();
     void CreateSettingsFile();
     bool set_default_filenames();
-    void parse_create_args( Timbl::TimblOpts& Opts );
-    void parse_run_args( Timbl::TimblOpts& Opts, bool = false );
+    bool parse_create_args( Timbl::TimblOpts& Opts );
+    bool parse_run_args( Timbl::TimblOpts& Opts, bool = false );
     bool isClone() const { return cloned; };
-    void ShowCats( std::ostream& os, std::vector<int>& Pat, int slots );
+    void ShowCats( std::ostream& os, const std::vector<int>& Pat, int slots );
     bool setLog( LogStream& );
     int ProcessLines( std::istream&, std::ostream& );
     void read_lexicon( const std::string& );
@@ -102,7 +110,6 @@ namespace Tagger {
     bool isInit() const { return initialized; };
   private:
     LogStream *cur_log;
-    sentence mySentence;
     Timbl::TimblAPI *KnownTree;
     Timbl::TimblAPI *unKnownTree;
     std::string Timbl_Options;
@@ -111,7 +118,6 @@ namespace Tagger {
     std::string unknownstr;
     std::string uwf;
     std::string kwf;
-    int nwords;
     bool initialized;
     StringHash TheLex;
     StringHash *kwordlist;
@@ -137,23 +143,26 @@ namespace Tagger {
     std::vector<std::string> distribution_array;
     std::vector<double> confidence_array;
 
+    void manifest();
     int makedataset( std::istream& infile, bool do_known );
     bool readsettings( std::string& fname );
     void create_lexicons();
     int ProcessFile( std::istream&, std::ostream& );
     void ProcessTags( TagInfo * );
-    std::string Tag();
-    void InitTest( MatchAction );
-    bool NextBest( int, int );
+    void InitTest( const sentence&, std::vector<int>&, MatchAction );
+    bool NextBest( const sentence&, std::vector<int>&, int, int );
     const Timbl::TargetValue *Classify( MatchAction, const std::string&, 
 					const Timbl::ValueDistribution **distribution, 
 					double& );
-    std::string get_result();
-    void statistics( int& no_known,
+    void statistics( const sentence&,
+		     int& no_known,
 		     int& no_unknown,
 		     int& no_correct_known, 
 		     int& no_correct_unknown );
-    std::string pat_to_string( MatchAction, int );
+    std::string pat_to_string( const sentence&, 
+			       const std::vector<int>&,
+			       MatchAction,
+			       int );
 
     std::string TimblOptStr;
     int FilterThreshold;
@@ -190,9 +199,32 @@ namespace Tagger {
     std::string SettingsFilePath;
     
     bool cloned;
-    std::vector<int> TestPat; 
   };
 
+  class TagResult {
+    friend std::vector<TagResult> TaggerClass::tagSentence( sentence& );
+  public:
+  TagResult(): _distance(-1), _confidence(-1), _known(false){};
+    bool isKnown() const { return _known; };
+    std::string word() const { return _word; };
+    std::string assignedTag() const { return _tag; };
+    std::string inputTag() const { return _inputTag; };
+    std::string enrichment() const { return _enrichment; };
+    std::string distribution() const { return _distribution; };
+    double confidence() const { return _confidence; };
+    double distance() const { return _distance; };
+  private:
+    std::string _word;
+    std::string _inputTag;
+    std::string _tag;
+    std::string _enrichment;
+    std::string _distribution;
+    double _distance;
+    double _confidence;
+    bool _known;
+  };
+  
+
   void RemoveTagger( TaggerClass * );
 }
 
