@@ -1,7 +1,7 @@
 /*
-  $Id: MbtAPI.cxx 13736 2011-12-20 16:51:18Z sloot $
+  $Id: MbtAPI.cxx 14402 2012-03-07 14:04:57Z sloot $
   $URL: https://ilk.uvt.nl/svn/trunk/sources/Mbt3/src/MbtAPI.cxx $
-  Copyright (c) 1998 - 2011
+  Copyright (c) 1998 - 2012
   ILK   - Tilburg University
   CLiPS - University of Antwerp
  
@@ -29,6 +29,7 @@
 #include <iostream>
 #include <ctime>
 #include <cstdlib>
+#include <stdexcept> 
 #include "timbl/TimblAPI.h"
 #include "config.h"
 #include "mbt/Logging.h"
@@ -40,6 +41,7 @@ using std::cout;
 using std::cerr;
 using std::endl;
 using std::string;
+using std::vector;
 
 MbtAPI::MbtAPI( const std::string& opts ){
   // get all the commandline options in an TimblOpts structure
@@ -67,8 +69,23 @@ string MbtAPI::Tag( const std::string& inp ){
   if ( tagger )
     return tagger->Tag( inp );
   else {
-    cerr << "No tagger initialized yet...." << endl;
-    exit(EXIT_FAILURE);
+    throw std::runtime_error( "No tagger initialized yet...." );
+  }
+}
+
+vector<TagResult> MbtAPI::TagLine( const string& inp ){
+  if ( tagger )
+    return tagger->tagLine( inp );
+  else {
+    throw std::runtime_error( "No tagger initialized yet...." );
+  }
+}
+
+string MbtAPI::getResult( const vector<TagResult>& v ) const {
+  if ( tagger )
+    return tagger->TRtoString( v );
+  else {
+    throw std::runtime_error( "No tagger initialized yet...." );
   }
 }
 
@@ -115,8 +132,7 @@ bool MbtAPI::GenerateTagger( TimblOpts& Opts ) {
   time_t timebefore, timeafter, timediff;
   time(&timebefore);
   int nw = TaggerClass::CreateTagger( Opts );
-  if ( nw <= 0 ){
-    cerr << "Tagger Creation failed" << endl;
+  if ( nw < 0 ){
     return false;
   }
   time(&timeafter);
@@ -152,6 +168,7 @@ void run_usage( char *progname ){
        << "\t-r <ambitagfile>\n"
        << "\t-k <known words case base>\n"
        << "\t-u <unknown words case base>\n"
+       << "\t-D <loglevel> (possible values are 'LogNormal', 'LogDebug', 'LogHeavy' and 'LogExtreme')\n"
        << "\t-e <sentence delimiter> (default '<utt>')\n"
        << "\t-E <enriched tagged testfile>\n "
        << "\t-t <testfile> | -T <tagged testfile> "
@@ -186,7 +203,6 @@ bool MbtAPI::RunTagger( int argc, char **argv ){
   time(&timebefore);
   TaggerClass *tagger = TaggerClass::StartTagger( Opts );
   if ( !tagger ){
-    cerr << "Starting Tagger failed" << endl;
     return false;
   }
   int nw = tagger->Run();
