@@ -1,8 +1,8 @@
 /*
-  $Id: Sentence.cxx 9077 2011-03-23 10:11:02Z sloot $
+  $Id: Sentence.cxx 14420 2012-03-07 16:54:30Z sloot $
   $URL: https://ilk.uvt.nl/svn/trunk/sources/Mbt3/src/Sentence.cxx $
 
-  Copyright (c) 1998 - 2011
+  Copyright (c) 1998 - 2012
   ILK   - Tilburg University
   CLiPS - University of Antwerp
  
@@ -52,12 +52,12 @@ namespace Tagger {
   // New enriched word.
   //
   word::word( const string& some_word, const vector<string>& extra_features, const string& some_tag){
-      the_word =  some_word;
-      word_tag = some_tag;
-      the_word_index = -1;
-      extraFeatures = extra_features;
+    the_word =  some_word;
+    word_tag = some_tag;
+    the_word_index = -1;
+    extraFeatures = extra_features;
   }  
- 
+  
   // Delete a word
   //
   word::~word(){
@@ -116,15 +116,6 @@ namespace Tagger {
     os << "'";
   }
   
-  void sentence::reset( const string& EosMark ){
-    // cleanup the sentence for re-use...
-    for ( unsigned int i=0; i < no_words; i++ )
-      delete Words[i];
-    Words.clear();
-    no_words = 0;
-    InternalEosMark = EosMark;
-  }
-  
   bool sentence::Utt_Terminator( const string& z_something ){
     return ( z_something == InternalEosMark );
   }
@@ -149,7 +140,7 @@ namespace Tagger {
   bool sentence::init_windowing( PatTemplate *Ktmpl,
 				 PatTemplate *Utmpl, 
 				 Lexicon &lex,
-				 StringHash& TheLex ){
+				 StringHash& TheLex ) {
     if ( UTAG == -1 )
       UTAG = TheLex.Hash( UNKNOWN );
     if ( no_words == 0 ) {
@@ -181,7 +172,7 @@ namespace Tagger {
     }
   }
   
-  int sentence::classify_hapax( const string& word, StringHash& TheLex ){
+  int sentence::classify_hapax( const string& word, StringHash& TheLex ) const{
     string hap = "HAPAX-";
     if ( word.find( "-" ) != string::npos ) // hyphen anywere
       hap += 'H';
@@ -197,7 +188,7 @@ namespace Tagger {
   
   bool sentence::nextpat( MatchAction *Action, vector<int>& Pat, 
 			  StringHash& wordlist, StringHash& TheLex,
-			  unsigned int position, int *old_pat ) {
+			  unsigned int position, int *old_pat ) const {
     // safety check:
     //
     if( no_words == 0 || position >= no_words)
@@ -387,7 +378,7 @@ namespace Tagger {
       Words[pos]->word_ass_tag = cat;
   }
   
-  bool sentence::known( unsigned int i ){
+  bool sentence::known( unsigned int i ) const {
     if( no_words > 0 && i < no_words )
       return Words[i]->word_amb_tag != UTAG;
     else
@@ -432,36 +423,17 @@ namespace Tagger {
   }
 
 
-  bool sentence::read( istream &infile, input_kind_type kind, bool lineMode ){
+  bool sentence::read( istream &infile, input_kind_type kind, 
+		       const string& eom ){
+    InternalEosMark = eom;
     if ( kind == TAGGED ||
 	 kind == UNTAGGED ){
-      if ( lineMode )
-	return readLine( infile, kind == TAGGED );
-      else
-	return read( infile, kind == TAGGED );
+      return read( infile, kind == TAGGED );
     }
     else
-      return read( infile );
+      return read_enriched( infile );
   }
 
-  bool sentence::readLine( istream &infile, bool tagged ){
-    // read a sentence from a stream
-    // every sentence is limited to one line
-    // be aware of \r\n problems
-    string linebuffer = "";
-    if ( getline( infile, linebuffer ) ){
-      if ( linebuffer.length() > 0 ){
-	if ( linebuffer[linebuffer.length()-1] == '\r' ){
-	  linebuffer.erase( linebuffer.length()-1 );
-	}
-	if ( linebuffer.length() > 0 ){
-	  return fill( linebuffer, tagged );
-	}
-      }
-    }
-    return false;
-  }
-  
   bool sentence::read( istream &infile, bool tagged ){
     // read a whole sentence from a stream
     // A sentence can be delimited either by an Eos marker or EOF.
@@ -504,23 +476,28 @@ namespace Tagger {
       return false;
   }
   
-  bool sentence::read( istream &infile ){
+  bool sentence::read_enriched( istream &infile ){
     // read an enriched and tagged word from infile
     // it must be a one_liner
     //    cerr << "Reading enriched" << endl;
+    // cleanup the sentence for re-use...
+    for ( unsigned int i=0; i < no_words; i++ )
+      delete Words[i];
+    Words.clear();
+    no_words = 0;
     string linebuffer;
     string Word;
     string Tag;
     vector<string> extras;
-    bool nothing_yet = true;
-    while( nothing_yet && infile ) {
+    bool not_ready = true;
+    while( not_ready && infile ) {
       getline( infile, linebuffer );
       infile >> ws;
       size_t size = split( linebuffer, extras );
       if ( size == 1 && Utt_Terminator( extras.front() ) ){
-	Word = extras.front();
-	add( Word, "" );
-	nothing_yet = false;
+	// Word = extras.front();
+	// add( Word, "" );
+	not_ready = false;
       }
       if ( size >= 2 ){
 	Word = extras.front();
@@ -530,13 +507,18 @@ namespace Tagger {
 	if ( !Word.empty() && !Tag.empty() ){
 	  add( Word, extras, Tag );
 	}
-	nothing_yet = false;
+	//	nothing_yet = false;
       }
     };
     return no_words > 0;
   }
   
   bool sentence::fill( const string& line, bool tagged ){
+    // cleanup the sentence for re-use...
+    for ( unsigned int i=0; i < no_words; i++ )
+      delete Words[i];
+    Words.clear();
+    no_words = 0;
     string token,  tagtoken;
     bool result = true;
 
