@@ -1,8 +1,8 @@
 /*
-  $Id: Tagger.cxx 13736 2011-12-20 16:51:18Z sloot $
+  $Id: Tagger.cxx 14421 2012-03-08 10:33:03Z sloot $
   $URL: https://ilk.uvt.nl/svn/trunk/sources/Mbt3/src/Tagger.cxx $
 
-  Copyright (c) 1998 - 2011
+  Copyright (c) 1998 - 2012
   ILK   - Tilburg University
   CLiPS - University of Antwerp
  
@@ -52,6 +52,10 @@
 #endif
 
 LogStream default_log( std::cerr ); // fall-back
+LogStream default_cout( std::cout, "", NoStamp ); // guard cout too
+
+#define COUT *Log(default_cout)
+
 LogStream *cur_log = &default_log;  // fill the externals
 
 LogLevel internal_default_level = LogNormal;
@@ -73,6 +77,7 @@ namespace Tagger {
     cur_log = new LogStream( cerr );
     cur_log->setlevel( LogNormal );
     cur_log->setstamp( StampMessage );
+    default_cout.setstamp( NoStamp );
     KnownTree = NULL;
     unKnownTree = NULL;
     TimblOptStr = "+vS -FColumns K: -a IGTREE +D U: -a IB1 ";
@@ -482,20 +487,6 @@ namespace Tagger {
     return Beam->Init( Beam_Size, no_words );
   }
 
-  int TaggerClass::TagLine( const string& line, string& result ){
-    result.clear();
-    mySentence.reset( EosMark );
-    mySentence.fill( line, input_kind );
-    //    LOG << "read a sentence " << mySentence << endl;
-    if ( mySentence.size() != 0 ){
-      result = Tag();
-      if ( !result.empty() ) {
-	//	LOG << "tagged a sentence " << result << endl;
-      }
-    } 
-    return mySentence.size();
-  }
-  
   int TaggerClass::ProcessLines( istream &is, ostream& os ){
     int no_words=0;    
     // loop as long as you get non empty sentences
@@ -503,10 +494,11 @@ namespace Tagger {
     string tagged_sentence;
     string line;
     while ( getline(is, line ) ){
-      int num = TagLine( line, tagged_sentence );
+      vector<TagResult> res = tagLine( line );
+      int num = res.size();
       if ( num > 0 ){
 	no_words += num;
-	os << tagged_sentence << endl;
+	os << TRtoString( res ) << endl;
       }
     } // end of while looping over sentences
     cerr << endl << "Done:" << endl
@@ -601,12 +593,12 @@ namespace Tagger {
   
   void prefixWithAbsolutePath( string& fileName, 
 			       const string& prefix ) {
-    //    cout << fileName << endl;
+    //    default_log << fileName << endl;
     if ( ( fileName[0] != '/' && fileName[1] != ':' )
 	 && !( fileName[0]== '.' && fileName[1] == '/' ) ){
       fileName = prefix + fileName;
     }
-    //    cout << fileName << endl;
+    //    default_log << fileName << endl;
   }
     
   bool TaggerClass::set_default_filenames( ){
@@ -789,7 +781,7 @@ namespace Tagger {
     }
   }
   
-  void TaggerClass::ShowCats( ostream& os, vector<int>& Pat, int slots ){
+  void TaggerClass::ShowCats( ostream& os, const vector<int>& Pat, int slots ){
     os << "Pattern : ";
     for( int slot=0; slot < slots; slot++){
       os << indexlex( Pat[slot], TheLex )<< " ";
@@ -797,7 +789,10 @@ namespace Tagger {
     os << endl;
   }
   
-  string TaggerClass::pat_to_string( MatchAction action, int word ){
+  string TaggerClass::pat_to_string( const sentence& mySentence,
+				     const vector<int>& pat,
+				     MatchAction action, 
+				     int word ){
     int slots;
     if ( action == Unknown )
       slots = Utemplate.totalslots() - Utemplate.skipfocus;
@@ -805,7 +800,7 @@ namespace Tagger {
       slots = Ktemplate.totalslots() - Ktemplate.skipfocus;
     string line;
     for( int f=0; f < slots; f++ ){
-      line += indexlex( TestPat[f], TheLex );
+      line += indexlex( pat[f], TheLex );
       line += " ";
     }
     for (vector<string>::iterator it = mySentence.getWord(word)->extraFeatures.begin(); it != mySentence.getWord(word)->extraFeatures.end(); it++)
@@ -818,13 +813,13 @@ namespace Tagger {
     else
       line += "??";
     if ( IsActive(DBG) ){
-      ShowCats( LOG, TestPat, slots );
+      ShowCats( LOG, pat, slots );
     }
     // dump if desired
     //
     if ( dumpflag ){
       for( int slot=0; slot < slots; slot++){
-	cout << indexlex( TestPat[slot], TheLex );
+	cout << indexlex( pat[slot], TheLex );
       }
       cout << endl;
     }
@@ -896,7 +891,6 @@ namespace Tagger {
     //
     read_listfile( TopNFileName, kwordlist );
     
-    nwords = 0;
     if ( TimblOptStr.empty() )
       Timbl_Options = "-FColumns ";
     else
@@ -982,14 +976,12 @@ namespace Tagger {
     // the testpattern is of the form given in Ktemplate and Utemplate
     // here we allocate enough space for the larger of them to serve both
     //
-    TestPat.reserve(max(Ktemplate.totalslots(),Utemplate.totalslots()));
     initialized = true;
     return true;
   }
 
   TaggerClass *TaggerClass::clone() const {
     TaggerClass *ta = new TaggerClass( *this );
-    ta->TestPat.reserve(max(Ktemplate.totalslots(),Utemplate.totalslots()));
     ta->Beam = NULL; // own Beaming data
     ta->cloned = true;
     return ta;
@@ -1003,7 +995,7 @@ namespace Tagger {
 	os = new ofstream( OutputFileName.c_str() );
       }
       else
-	os = &cout;
+	os = &default_cout;
       ifstream infile;    
       if ( !piped_input ){
 	string inname = TestFilePath + TestFileName;
@@ -1064,9 +1056,11 @@ namespace Tagger {
     return answer;
   }
 
-  void TaggerClass::InitTest( MatchAction Action ){
+  void TaggerClass::InitTest( const sentence& mySentence, 
+			      vector<int>& TestPat, 
+			      MatchAction Action ){
     // Now make a testpattern for Timbl to process.
-    string teststring = pat_to_string( Action, 0 );
+    string teststring = pat_to_string( mySentence, TestPat, Action, 0 );
     const ValueDistribution *distribution = 0;
     double distance;
     const TargetValue *answer = Classify( Action, teststring, &distribution, distance );
@@ -1092,16 +1086,18 @@ namespace Tagger {
   }
   
   
-  bool TaggerClass::NextBest( int i_word, int beam_cnt ){
+  bool TaggerClass::NextBest( const sentence& mySentence,
+			      vector<int>& TestPat,
+			      int i_word, int beam_cnt ){
     MatchAction Action = Unknown;
     if ( Beam->paths[beam_cnt][i_word-1] == EMPTY_PATH ){
       return false;
     }
     else if ( mySentence.nextpat( &Action, TestPat,
-				   *kwordlist, TheLex,
-				   i_word, Beam->paths[beam_cnt] ) ){
+				  *kwordlist, TheLex,
+				  i_word, Beam->paths[beam_cnt] ) ){
       // Now make a testpattern for Timbl to process.
-      string teststring = pat_to_string( Action, i_word );
+      string teststring = pat_to_string( mySentence, TestPat, Action, i_word );
       // process teststring to predict a category, using the 
       // appropriate tree
       //
@@ -1130,64 +1126,88 @@ namespace Tagger {
     }
   }
   
-  string TaggerClass::Tag( const string& inp ){
-    string result;
-    TagLine( inp, result );
-    return result;
+  int TaggerClass::TagLine( const std::string& inp, string& result ){
+    vector<TagResult> res = tagLine( inp );
+    result = TRtoString( res );
+    return res.size();
   }
 
-  string TaggerClass::Tag(){
-    string tag = "";
-    if ( !initialized ||
-	 !InitBeaming( mySentence.size() ) ){
-      throw runtime_error( "Tagger not initialized" );
-      return tag;
-    }
-    DBG << mySentence << endl;
-    
-    if ( mySentence.init_windowing(&Ktemplate,&Utemplate, 
-				   *MT_lexicon, TheLex ) ) {
-      // here the word window is looked up in the dictionary and the values
-      // of the features are stored in the testpattern
-      MatchAction Action = Unknown;
-      if ( mySentence.nextpat( &Action, TestPat, 
-			       *kwordlist, TheLex,
-			       0 )){ 
-	
-	DBG << "Start: " << mySentence.getword( 0 ) << endl;
-	InitTest( Action );
-	for ( unsigned int iword=1; iword < mySentence.size(); iword++ ){
-	  // clear best_array
-	  DBG << endl << "Next: " << mySentence.getword( iword ) << endl;
-	  Beam->ClearBest();
-	  for ( int beam_count=0; beam_count < Beam_Size; beam_count++ ){
-	    if ( !NextBest( iword, beam_count ) )
-	      break;
+  vector<TagResult> TaggerClass::tagLine( const string& line ){
+    vector<TagResult> result;
+    sentence mySentence;
+    mySentence.fill( line, input_kind );
+    return tagSentence( mySentence );
+  }
+
+  vector<TagResult> TaggerClass::tagSentence( sentence& mySentence ){
+    vector<TagResult> result;
+    if ( mySentence.size() != 0 ){
+      if ( !initialized ||
+	   !InitBeaming( mySentence.size() ) ){
+	throw runtime_error( "Tagger not initialized" );
+      }
+      DBG << mySentence << endl;
+      
+      if ( mySentence.init_windowing(&Ktemplate,&Utemplate, 
+				     *MT_lexicon, TheLex ) ) {
+	// here the word window is looked up in the dictionary and the values
+	// of the features are stored in the testpattern
+	MatchAction Action = Unknown;
+	vector<int> TestPat;
+	TestPat.reserve(Utemplate.totalslots());
+	if ( mySentence.nextpat( &Action, TestPat, 
+				 *kwordlist, TheLex,
+				 0 )){ 
+	  
+	  DBG << "Start: " << mySentence.getword( 0 ) << endl;
+	  InitTest( mySentence, TestPat, Action );
+	  for ( unsigned int iword=1; iword < mySentence.size(); iword++ ){
+	    // clear best_array
+	    DBG << endl << "Next: " << mySentence.getword( iword ) << endl;
+	    Beam->ClearBest();
+	    for ( int beam_count=0; beam_count < Beam_Size; beam_count++ ){
+	      if ( !NextBest( mySentence, TestPat, iword, beam_count ) )
+		break;
+	    }
+	    Beam->Shift( mySentence.size(), iword );
+	    if ( IsActive( DBG ) ){
+	      LOG << "after shift:" << endl;
+	      Beam->Print( LOG, iword, TheLex );
+	    }
 	  }
-	  Beam->Shift( mySentence.size(), iword );
-	  if ( IsActive( DBG ) ){
-	    LOG << "after shift:" << endl;
-	    Beam->Print( LOG, iword, TheLex );
-	  }
-	}
-      } // end one sentence
-      tag = get_result();
-    }
-    return tag;
+	} // end one sentence
+      }
+      // get output
+      for ( unsigned int Wcnt=0; Wcnt < mySentence.size(); ++Wcnt ){
+	TagResult res;
+	// get the original word
+	res._word= mySentence.getword(Wcnt);
+	// get the original tag
+	res._inputTag = mySentence.gettag(Wcnt);
+	// lookup the assigned tag
+	res._tag = indexlex( Beam->paths[0][Wcnt], TheLex );
+	// is it known/unknown
+	res._known = mySentence.known(Wcnt);
+	if ( input_kind == ENRICHED )
+	  res._enrichment = mySentence.getenr(Wcnt);
+	if ( confidence_flag )
+	  res._confidence = confidence_array[Wcnt];
+	if ( distrib_flag )
+	  res._distribution = distribution_array[Wcnt];
+	if ( distance_flag )
+	  res._distance = distance_array[Wcnt];
+	result.push_back( res );
+      }
+    } // end of output loop through one sentence
+    return result;
   }
   
-  string TaggerClass::get_result(){
+  string TaggerClass::TRtoString( const vector<TagResult>& tr ) const {
     string result;
-    string tagstring;
-    //now some output
-    for ( unsigned int Wcnt=0; Wcnt < mySentence.size(); ++Wcnt ){
+    for ( unsigned int Wcnt=0; Wcnt < tr.size(); ++Wcnt ){
       // lookup the assigned category
-      tagstring = indexlex( Beam->paths[0][Wcnt], TheLex );
-      // now we do the appropriate output, depending on known/unknown
-      // words and the availability of a "correct tag".
-      //
-      result += mySentence.getword(Wcnt);
-      if ( mySentence.known(Wcnt) ){
+      result += tr[Wcnt].word();
+      if ( tr[Wcnt].isKnown() ){
 	if ( input_kind == UNTAGGED )
 	  result += "/";
 	else
@@ -1201,40 +1221,33 @@ namespace Tagger {
       }      
       // output the correct tag if possible
       //
-      if ( input_kind == ENRICHED ){
-	result = result + mySentence.getenr(Wcnt) + "\t" 
-	  + mySentence.gettag(Wcnt) + "\t" + tagstring;
+      if ( input_kind == ENRICHED )
+	result = result + tr[Wcnt].enrichment() + "\t";
+      if ( input_kind == TAGGED ||
+	   input_kind == ENRICHED ){
+	result += tr[Wcnt].inputTag() + "\t" + tr[Wcnt].assignedTag();
 	if ( confidence_flag )
-	  result += " [" + toString( confidence_array[Wcnt] ) + "]";
+	  result += " [" + toString( tr[Wcnt].confidence() ) + "]";
 	if ( distrib_flag )
-	  result += " " + distribution_array[Wcnt];
+	  result += " " + tr[Wcnt].distribution();
 	if ( distance_flag )
-	  result += " " + toString( distance_array[Wcnt] );
-	result += "\n";
-      }
-      else if ( input_kind == TAGGED ){
-	result = result + mySentence.gettag(Wcnt) + "\t" + tagstring;
-	if ( confidence_flag )
-	  result += " [" + toString( confidence_array[Wcnt] ) + "]";
-	if ( distrib_flag )
-	  result += " " + distribution_array[Wcnt];
-	if ( distance_flag )
-	  result += " " + toString( distance_array[Wcnt] );
+	  result += " " + toString( tr[Wcnt].distance() );
 	result += "\n";
       }
       else {
-	result = result + tagstring;
+	result += tr[Wcnt].assignedTag();
 	if ( confidence_flag )
-	  result += "/" + toString( confidence_array[Wcnt] );
+	  result += "/" + toString( tr[Wcnt].confidence() );
 	result += " ";
       }
     } // end of output loop through one sentence
     if ( input_kind != ENRICHED )
-      result = result + mySentence.Eos();
+      result = result + EosMark;
     return result;
   }
-  
-  void TaggerClass::statistics( int& no_known, int& no_unknown,
+
+  void TaggerClass::statistics( const sentence& mySentence,
+				int& no_known, int& no_unknown,
 				int& no_correct_known, 
 				int& no_correct_unknown ){
     string result;
@@ -1270,8 +1283,9 @@ namespace Tagger {
     // loop as long as you get sentences
     //
     int HartBeat = 0;
+    sentence mySentence;
     while ( go_on && 
-	    (mySentence.reset( EosMark), mySentence.read(infile, input_kind ) ) ){
+	    mySentence.read(infile, input_kind, EosMark ) ){
       if ( mySentence.size() == 0 )
 	continue;
       string tagged_sentence;
@@ -1283,10 +1297,12 @@ namespace Tagger {
 	outfile << EosMark << endl;
 	continue;
       }
-      tagged_sentence = Tag();
+      vector<TagResult> res = tagSentence( mySentence );
+      tagged_sentence = TRtoString( res );
       if ( !tagged_sentence.empty() ){
 	// show the results of 1 sentence
-	statistics( no_known, no_unknown,
+	statistics( mySentence,
+		    no_known, no_unknown,
 		    no_correct_known, 
 		    no_correct_unknown );
 	outfile << tagged_sentence << endl;
@@ -1438,13 +1454,13 @@ namespace Tagger {
     return true;
   }
 
-  void TaggerClass::parse_run_args( TimblOpts& Opts, bool as_server ){
+  bool TaggerClass::parse_run_args( TimblOpts& Opts, bool as_server ){
     string value;
     bool mood;
     if ( Opts.Find( 'V', value, mood ) ||
 	 Opts.Find( "version", value, mood ) ){
-      // we already identified ourself. just bail out.
-      exit(EXIT_SUCCESS);
+      manifest();
+      return false;
     }
     if ( Opts.Find( 's', value, mood ) ){
       // if a settingsfile option has been given, read that first
@@ -1459,7 +1475,7 @@ namespace Tagger {
 	SettingsFilePath = "";
       if( !readsettings( SettingsFileName ) ){
 	cerr << "Cannot read settingsfile " << SettingsFileName << endl;
-	exit(EXIT_FAILURE);
+	return false;
       }
       Opts.Delete( 's' );
     };
@@ -1526,7 +1542,7 @@ namespace Tagger {
       cerr << "Server mode NOT longer supported in this version!\n"
 	   << "You must use mbtserver instead\n"
 	   << "sorry..." << endl;
-      exit(EXIT_FAILURE);
+      return false;
     };
     if ( Opts.Find( 't', value, mood ) ){
       TestFileName = value;
@@ -1545,7 +1561,7 @@ namespace Tagger {
       if ( input_kind == ENRICHED ){
 	cerr << "Option -T conflicts with ENRICHED format from settingsfile "
 	     << "unable to continue" << endl;
-	exit(EXIT_FAILURE);
+	return false;
       }
       input_kind = TAGGED; // there is a tagged test file specified
       Opts.Delete( 'T' );
@@ -1571,30 +1587,38 @@ namespace Tagger {
     if ( cloned && input_kind == ENRICHED ){
       cerr << "Servermode doesn't support enriched inputformat!" << endl
 	   << "bailing out, sorry " << endl;
-      exit(EXIT_FAILURE);
+      return false;
     }
     if ( !as_server &&
 	 (!knowntreeflag || !unknowntreeflag) ){
       cerr << "missing required options. See 'mbt -h' " << endl;
-      exit(EXIT_FAILURE);
+      return false;
     }
+    return true;
   }
-  
-  TaggerClass *TaggerClass::StartTagger( TimblOpts& Opts, LogStream* os ){
+
+  void TaggerClass::manifest( ){
     // present yourself to the user
     //
-    cerr << "mbt " << VERSION << " (c) ILK and CLiPS 1998 - 2011." << endl
-	 << "Memory Based Tagger " << endl
-	 << "Tilburg University" << endl
-	 << "CLiPS Computational Linguistics Group, University of Antwerp"
-	 << endl
-	 << "Based on " << Timbl::VersionName() 
-	 << endl << endl;
-    
+    LOG << "mbt " << VERSION << " (c) ILK and CLiPS 1998 - 2012." << endl
+	<< "Memory Based Tagger " << endl
+	<< "Tilburg University" << endl
+	<< "CLiPS Computational Linguistics Group, University of Antwerp"
+	<< endl
+	<< "Based on " << Timbl::VersionName() 
+	<< endl << endl;
+  }
+  
+  TaggerClass *TaggerClass::StartTagger( TimblOpts& Opts, LogStream* os ){    
     TaggerClass *tagger = new TaggerClass;
-    tagger->parse_run_args( Opts );
+    if ( !tagger->parse_run_args( Opts ) ){
+      delete tagger;
+      return 0;
+    }
     if ( os )
       tagger->setLog( *os );
+    else // only manifest() when running 'standalone'
+      tagger->manifest();
     tagger->set_default_filenames();
     tagger->InitTagging();
     return tagger;
@@ -1617,10 +1641,6 @@ namespace Tagger {
     splits( Timbl_Options, commonstr, knownstr, unknownstr );
     get_weightsfile_name( knownstr, kwf );
     get_weightsfile_name( unknownstr, uwf );
-    // the testpattern is of the form given in Ktemplate and Utemplate
-    // here we allocate enough space for the larger of them to serve both
-    //
-    TestPat.reserve(max(Ktemplate.totalslots(),Utemplate.totalslots()));
     return true;
   }
   
@@ -1630,20 +1650,24 @@ namespace Tagger {
     int nslots=0;
     ofstream outfile;
     MatchAction Action;
+    vector<int> TestPat;
     if( do_known ){
       nslots = Ktemplate.totalslots() - Ktemplate.skipfocus;
       outfile.open( K_option_name.c_str(), ios::trunc | ios::out );
       Action = MakeKnown;
+      TestPat.reserve(Ktemplate.totalslots());
     }
     else {
       nslots = Utemplate.totalslots() - Utemplate.skipfocus;
       outfile.open( U_option_name.c_str(), ios::trunc | ios::out );
       Action = MakeUnknown;
+      TestPat.reserve(Utemplate.totalslots());
     }
     // loop as long as you get sentences
     //
     int HartBeat = 0;
-    while ( (mySentence.reset( EosMark), mySentence.read( infile, input_kind ) )){
+    sentence mySentence;
+    while ( mySentence.read( infile, input_kind, EosMark ) ){
       if ( mySentence.size() == 0 )
 	continue;
       // cerr << mySentence << endl;
@@ -1651,19 +1675,26 @@ namespace Tagger {
 	// only possible for ENRICHED!
 	continue;
       }
-      if ( HartBeat++ % 100 == 0 ){
-	cout << "."; cout.flush();
+      if ( ++HartBeat % 100 == 0 ) {
+	if ( do_known ){
+	  COUT << "+";
+	  default_cout.flush();
+	}
+	else {
+	  COUT << "-";
+	  default_cout.flush();
+	}
       }
       if ( mySentence.init_windowing( &Ktemplate,&Utemplate, 
-				       *MT_lexicon, TheLex ) ) {
+				      *MT_lexicon, TheLex ) ) {
 	// we initialize the windowing procedure, this entails lexical lookup
 	// of the words in the dictionary and the values
 	// of the features are stored in the testpattern
 	int swcn = 0;
 	int thisTagCode;
 	while( mySentence.nextpat( &Action, TestPat, 
-				    *kwordlist, TheLex,
-				    swcn ) ){ 
+				   *kwordlist, TheLex,
+				   swcn ) ){ 
 	  bool skip = false;
 	  if ( DoNpax && !do_known ){
 	    if((uwordlist->Lookup(mySentence.getword(swcn)))==0){
@@ -1687,26 +1718,23 @@ namespace Tagger {
 	no_sentences++;
       }
     }
-    cout << endl << "      ready: " << no_words << " words processed." 
-	 << endl;
-//      cout << "Output written to ";
+//      default_cout << "Output written to ";
 //      if ( do_known )
-//        cout << K_option_name << endl;
+//        default_cout << K_option_name << endl;
 //      else 
-//        cout << U_option_name << endl;
+//        default_cout << U_option_name << endl;
     return no_words;
   }
   
   int TaggerClass::CreateKnown(){
     int nwords = 0;
     if ( knowntemplateflag ){
-      cout << endl << "  Create known words case base" << endl
-	   << "    Timbl options: '" << knownstr + commonstr << "'" << endl;
-      TimblAPI *UKtree = new TimblAPI( knownstr + commonstr );
-      if ( !UKtree->Valid() )
+      COUT << "  Create known words case base,"
+	   << "   Timbl options: '" << knownstr + commonstr << "'" << endl;
+      TimblAPI *Ktree = new TimblAPI( knownstr + commonstr );
+      if ( !Ktree->Valid() )
 	exit(EXIT_FAILURE);
-      cout << "    Algorithm = " << to_string(UKtree->Algo())
-	   << endl << endl;
+      COUT << "    Algorithm = " << to_string(Ktree->Algo()) << endl;
       if ( !piped_input ){
 	string inname = TestFilePath + TestFileName;
 	ifstream infile( inname.c_str(), ios::in );
@@ -1714,24 +1742,24 @@ namespace Tagger {
 	  cerr << "Cannot read from " << inname << endl;
 	  return 0;
 	}
-	cout << "    Processing data from the file " << inname << "...";
-	cout.flush();
+	COUT << "    Processing data from the file " << inname << "...";
+	default_cout.flush();
 	nwords = makedataset( infile, true );
       }
       else {
-	cout << "Processing data from the standard input" << endl;
+	COUT << "Processing data from the standard input" << endl;
 	nwords = makedataset( cin, true );
       }
-      cout << "    Creating case base: " << KnownTreeName << endl;
-      UKtree->Learn( K_option_name );
-      //      UKtree->ShowSettings( cout );
-      UKtree->WriteInstanceBase( KnownTreeName );
+      COUT << endl << "    Creating case base: " << KnownTreeName << endl;
+      Ktree->Learn( K_option_name );
+      //      UKtree->ShowSettings( default_cout );
+      Ktree->WriteInstanceBase( KnownTreeName );
       if ( !kwf.empty() )
-	UKtree->SaveWeights( kwf );
-      delete UKtree;
+	Ktree->SaveWeights( kwf );
+      delete Ktree;
       if ( !KeepIntermediateFiles ){
 	remove(K_option_name.c_str());
-	cout << "    Deleted intermediate file: " << K_option_name << endl;
+	COUT << "    Deleted intermediate file: " << K_option_name << endl;
       }
     }
     return nwords;
@@ -1740,13 +1768,12 @@ namespace Tagger {
   int TaggerClass::CreateUnknown(){
     int nwords = 0;
     if ( unknowntemplateflag ){
-      cout << endl << "  Create unknown words case base" << endl
-	   << "    Timbl options: '" << unknownstr + commonstr << "'" << endl;
+      COUT << "  Create unknown words case base,"
+	   << " Timbl options: '" << unknownstr + commonstr << "'" << endl;
       TimblAPI *UKtree = new TimblAPI( unknownstr + commonstr );
       if ( !UKtree->Valid() )
 	exit(EXIT_FAILURE);
-      cout << "    Algorithm = " << to_string(UKtree->Algo()) 
-	   << endl << endl;
+      COUT << "    Algorithm = " << to_string(UKtree->Algo()) << endl;
       if ( !piped_input ){
 	string inname = TestFilePath + TestFileName;
 	ifstream infile( inname.c_str(), ios::in );
@@ -1754,24 +1781,24 @@ namespace Tagger {
 	  cerr << "Cannot read from " << inname << endl;
 	  return 0;
 	}
-	cout << "    Processing data from the file " << inname << "...";
-	cout.flush();
+	// default_cout << "    Processing data from the file " << inname << "...";
+	// default_cout.flush();
 	nwords = makedataset( infile, false );
       }
       else {
-	cout << "Processing data from the standard input" << endl;
+	COUT << "Processing data from the standard input" << endl;
 	nwords = makedataset( cin, false );
       }
-      cout << "    Creating case base: " << UnknownTreeName << endl;
+      COUT << endl << "    Creating case base: " << UnknownTreeName << endl;
       UKtree->Learn( U_option_name );
-      //      UKtree->ShowSettings( cout );
+      //      UKtree->ShowSettings( default_cout );
       UKtree->WriteInstanceBase( UnknownTreeName );
       if ( !uwf.empty() )
 	UKtree->SaveWeights( uwf );
       delete UKtree;
       if ( !KeepIntermediateFiles ){
 	remove(U_option_name.c_str());
-	cout << "    Deleted intermediate file: " << U_option_name << endl;
+	COUT << "    Deleted intermediate file: " << U_option_name << endl;
       }
     }
     return nwords;
@@ -1801,19 +1828,19 @@ namespace Tagger {
       out_file << "L " << TopNFileName << endl;
       out_file.close();
       cout << endl << "  Created settings file '" 
-	   << SettingsFileName << "'" << endl;
+			 << SettingsFileName << "'" << endl;
     }
   }
 
   //**** stuff to process commandline options *****************************
   
-  void TaggerClass::parse_create_args( TimblOpts& Opts ){
+  bool TaggerClass::parse_create_args( TimblOpts& Opts ){
     string value;
     bool mood;
     if ( Opts.Find( 'V', value, mood ) ||
 	 Opts.Find( "version", value, mood ) ){
       // we already identified ourself. just bail out.
-      exit(EXIT_SUCCESS);
+      return false;
     }
     if ( Opts.Find( '%', value, mood ) ){
       FilterThreshold = stringTo<int>( value );
@@ -1926,15 +1953,16 @@ namespace Tagger {
     }
     if ( TestFileName.empty() ){
       cerr << "Missing required options. see 'mbt -h'" << endl;
-      exit(EXIT_SUCCESS);
+      return false;
     }
+    return true;
   }
   
   int TaggerClass::CreateTagger( TimblOpts& Opts ){
     // 
     // present yourself to the user
     //
-    cerr << "mbtg " << VERSION << " (c) ILK and CLiPS 1998 - 2011." << endl
+    cerr << "mbtg " << VERSION << " (c) ILK and CLiPS 1998 - 2012." << endl
 	 << "Memory Based Tagger Generator" << endl
 	 << "Induction of Linguistic Knowledge Research Group,"
 	 << "Tilburg University" << endl
@@ -1943,16 +1971,29 @@ namespace Tagger {
 	 << "Based on " << Timbl::VersionName() 
 	 << endl << endl;
     TaggerClass tagger;
-    tagger.parse_create_args( Opts );
+    if ( !tagger.parse_create_args( Opts ) )
+      return -1;
     tagger.set_default_filenames();
     tagger.InitLearning();
     // process the test material
     // and do the timing
-    int nwords = 0;
-    nwords += tagger.CreateKnown();
-    nwords += tagger.CreateUnknown();
+    int kwords = 0;
+    int uwords = 0;
+#pragma omp parallel sections
+    {
+#pragma omp section
+      {
+	kwords = tagger.CreateKnown();
+      }
+#pragma omp section
+      {
+	uwords = tagger.CreateUnknown();
+      }
+    }
+    cout << "      ready: " << kwords << " words processed." 
+	 << endl;
     tagger.CreateSettingsFile();
-    return nwords;
+    return kwords + uwords;
   }  
 
 }
