@@ -1,8 +1,8 @@
 /*
-  $Id: simpletest.cxx 13164 2011-09-14 09:25:24Z sloot $
+  $Id: simpletest.cxx 14374 2012-03-05 11:13:09Z sloot $
   $URL: https://ilk.uvt.nl/svn/trunk/sources/Mbt3/src/simpletest.cxx $
 
-  Copyright (c) 1998 - 2011
+  Copyright (c) 1998 - 2012
   ILK   - Tilburg University
   CLiPS - University of Antwerp
  
@@ -35,6 +35,9 @@ int main(){
   string path = getenv( "topsrcdir" );
   MbtAPI::GenerateTagger( "-T " + path + "/example/eindh.data -s ./simple.setting " );
   MbtAPI demo( "-s ./simple.setting" );
-  cerr << demo.Tag( "dit is een test" );
+  cerr << demo.Tag( "dit is een test" ) << endl;
+  vector<TagResult> v = demo.TagLine( "Test regel 2 ." );
+  assert( v[0].assignedTag() == "N" );
+  assert( v[2].confidence() == -1 );
 }
 
